@@ -28,6 +28,7 @@ Here is a simple example of use of this module::
     <String: lo>
 """
 
+import inspect
 from time import time
 from collections import MutableMapping
 from snimpy import snmp, mib, basictypes
@@ -133,6 +134,16 @@ class CachedSession(DelegatedSession):
         self.count = 0
 
 
+def MibRestrictedManager(original, mibs):
+
+    """Copy an existing manager but restrict its view to the given set of
+    MIBs.
+    """
+    clone = Manager(**original._constructor_args)
+    clone._loaded = mibs
+    return clone
+
+
 class Manager(object):
 
     """SNMP manager. An instance of this class will represent an SNMP
@@ -198,7 +209,7 @@ class Manager(object):
                  community="public", version=2,
                  cache=False, none=False,
                  timeout=None, retries=None,
-                 loose=False,
+                 loose=False, bulk=40,
                  # SNMPv3
                  secname=None,
                  authprotocol=None, authpassword=None,
@@ -236,6 +247,9 @@ class Manager(object):
             instead of an exception. This mode should be enabled with
             caution. Patching the MIB is a better idea.
         :type loose: bool
+        :param bulk: Max-repetition to use to speed up MIB walking
+            with `GETBULK`. Set to `0` to disable.
+        :type bulk: int
         """
         if host is None:
             host = Manager._host
@@ -243,7 +257,8 @@ class Manager(object):
         self._session = snmp.Session(host, community, version,
                                      secname,
                                      authprotocol, authpassword,
-                                     privprotocol, privpassword)
+                                     privprotocol, privpassword,
+                                     bulk=bulk)
         if timeout is not None:
             self._session.timeout = int(timeout * 1000000)
         if retries is not None:
@@ -256,9 +271,18 @@ class Manager(object):
         if none:
             self._session = NoneSession(self._session)
         self._loose = loose
+        self._loaded = loaded
+
+        # To be able to clone, we save the arguments provided to the
+        # constructor in a generic way
+        frame = inspect.currentframe()
+        args, _, _, values = inspect.getargvalues(frame)
+        self._constructor_args = dict((a, values[a])
+                                      for a in args
+                                      if a != 'self')
 
     def _locate(self, attribute):
-        for m in loaded:
+        for m in self._loaded:
             try:
                 a = mib.get(m, attribute)
                 return (m, a)
@@ -295,6 +319,13 @@ class Manager(object):
             return
         raise AttributeError("{0} is not writable".format(attribute))
 
+    def __getitem__(self, modulename):
+        modulename = modulename.encode('ascii')
+        for m in loaded:
+            if modulename == m:
+                return MibRestrictedManager(self, [m])
+        raise KeyError("{0} is not a loaded module".format(modulename))
+
     def __repr__(self):
         return "<Manager for {0}>".format(self._host)
 
