@@ -26,8 +26,12 @@ incomplete.
 .. _CFFI: http://cffi.readthedocs.org/
 """
 
-from snimpy.smi import smi as _smi
-from snimpy.smi import ffi
+try:
+    from snimpy._smi import lib as _smi
+    from snimpy._smi import ffi
+except ImportError:
+    from snimpy.smi_build import ffi, get_lib
+    _smi = get_lib()
 
 
 class SMIException(Exception):
@@ -65,6 +69,8 @@ class Node(object):
             t = self._override_type
         else:
             t = _smi.smiGetNodeType(self.node)
+        if t == ffi.NULL:
+            raise SMIException("unable to retrieve type of node")
         target = {
             _smi.SMI_BASETYPE_INTEGER32: basictypes.Integer,
             _smi.SMI_BASETYPE_INTEGER64: basictypes.Integer,
