@@ -29,6 +29,7 @@ type coercing.
 import re
 import socket
 import inspect
+import threading
 from pysnmp.entity.rfc3413.oneliner import cmdgen
 from pysnmp.proto import rfc1902, rfc1905
 from pysnmp.smi import error
@@ -74,7 +75,7 @@ class Session(object):
     session. From such an instance, one can get information from the
     associated agent."""
 
-    _cmdgen = cmdgen.CommandGenerator()
+    _tls = threading.local()
 
     def __init__(self, host,
                  community="public", version=2,
@@ -82,7 +83,8 @@ class Session(object):
                  authprotocol=None,
                  authpassword=None,
                  privprotocol=None,
-                 privpassword=None):
+                 privpassword=None,
+                 bulk=40):
         """Create a new SNMP session.
 
         :param host: The hostname or IP address of the agent to
@@ -113,9 +115,18 @@ class Session(object):
         :param privpassword: Privacy password if privacy protocol is
             not `None`.
         :type privpassword: str
+        :param bulk: Max repetition value for `GETBULK` requests. Set
+            to `0` to disable.
+        :type bulk: int
         """
         self._host = host
         self._version = version
+        if version == 3:
+            self._cmdgen = cmdgen.CommandGenerator()
+        else:
+            if not hasattr(self._tls, "cmdgen"):
+                self._tls.cmdgen = cmdgen.CommandGenerator()
+            self._cmdgen = self._tls.cmdgen
 
         # Put authentication stuff in self._auth
         if version in [1, 2]:
@@ -191,7 +202,7 @@ class Session(object):
                                                               port))
 
         # Bulk stuff
-        self.bulk = 40
+        self.bulk = bulk
 
     def _check_exception(self, value):
         """Check if the given ASN1 value is an exception"""
@@ -241,6 +252,9 @@ class Session(object):
         else:
             results = [(tuple(name), val)
                        for row in varBinds for name, val in row]
+            if len(results) > 0 and isinstance(results[-1][1],
+                                               rfc1905.EndOfMibView):
+                results = results[:-1]
         if len(results) == 0:
             if cmd not in [self._cmdgen.nextCmd, self._cmdgen.bulkCmd]:
                 raise SNMPException("empty answer")
