@@ -9,25 +9,32 @@ from pysnmp.proto.api import v2c
 
 class TestAgent(object):
 
+    next_port = [random.randint(22000, 32000)]
+
     """Agent for testing purpose"""
 
-    def __init__(self, ipv6=False):
+    def __init__(self, ipv6=False, community='public',
+                 authpass='authpass', privpass='privpass'):
         q = Queue()
         self.ipv6 = ipv6
-        self._process = Process(target=self._setup, args=(q,))
+        self.community = community
+        self.authpass = authpass
+        self.privpass = privpass
+        self.next_port[0] += 1
+        self._process = Process(target=self._setup,
+                                args=(q, self.next_port[0]))
         self._process.start()
         self.port = q.get()
 
     def terminate(self):
         self._process.terminate()
 
-    def _setup(self, q):
+    def _setup(self, q, port):
         """Setup a new agent in a separate process.
 
         The port the agent is listening too will be returned using the
         provided queue.
         """
-        port = random.randrange(22000, 22989)
         snmpEngine = engine.SnmpEngine()
         if self.ipv6:
             config.addSocketTransport(
@@ -40,16 +47,15 @@ class TestAgent(object):
                 udp.domainName,
                 udp.UdpTransport().openServerMode(('127.0.0.1', port)))
         # Community is public and MIB is writable
-        config.addV1System(snmpEngine, 'read-write', 'public')
+        config.addV1System(snmpEngine, 'read-write', self.community)
         config.addVacmUser(snmpEngine, 1, 'read-write', 'noAuthNoPriv',
                            (1, 3, 6), (1, 3, 6))
         config.addVacmUser(snmpEngine, 2, 'read-write', 'noAuthNoPriv',
                            (1, 3, 6), (1, 3, 6))
         config.addV3User(
             snmpEngine, 'read-write',
-            config.usmHMACMD5AuthProtocol, 'authpass',
-            config.usmAesCfb128Protocol, 'privpass'
-        )
+            config.usmHMACMD5AuthProtocol, self.authpass,
+            config.usmAesCfb128Protocol, self.privpass)
         config.addVacmUser(snmpEngine, 3, 'read-write', 'authPriv',
                            (1, 3, 6), (1, 3, 6))
 
@@ -86,7 +92,9 @@ class TestAgent(object):
             # SNMPv2-MIB::sysDescr
             MibScalar((1, 3, 6, 1, 2, 1, 1, 1), v2c.OctetString()),
             MibScalarInstance((1, 3, 6, 1, 2, 1, 1, 1), (0,),
-                              v2c.OctetString("Snimpy Test Agent")))
+                              v2c.OctetString(
+                                  "Snimpy Test Agent {}".format(
+                                      self.community))))
         mibBuilder.exportSymbols(
             '__MY_IF_MIB',
             # IF-MIB::ifNumber
