@@ -1,8 +1,13 @@
-import unittest
+import sys
+if sys.version_info < (2, 7):
+    import unittest2 as unittest
+else:
+    import unittest
 import os
 import tempfile
 import code  # nopep8
 import mock
+import platform
 from snimpy.main import interact
 from multiprocessing import Process
 import agent
@@ -20,6 +25,8 @@ class TestMain(unittest.TestCase):
     def tearDownClass(cls):
         cls.agent.terminate()
 
+    @unittest.skipIf(platform.python_implementation() == "PyPy",
+                     "setupterm seems unreliable with Pypy")
     def test_loadfile(self):
         script = tempfile.NamedTemporaryFile(delete=False)
         try:
