@@ -1,5 +1,7 @@
 import unittest
 import os
+import threading
+import multiprocessing
 from datetime import timedelta
 from snimpy import basictypes, snmp, mib
 import agent
@@ -124,28 +126,42 @@ class TestSnmp1(unittest.TestCase):
     """
     version = 1
 
+    @classmethod
+    def addAgent(cls, community, auth, priv):
+        a = agent.TestAgent(community=community,
+                            authpass=auth,
+                            privpass=priv)
+        cls.agents.append(a)
+        return a
+
     @classmethod
     def setUpClass(cls):
         mib.load('IF-MIB')
         mib.load('SNMPv2-MIB')
-        cls.agent = agent.TestAgent()
+        cls.agents = []
+        cls.agent = cls.addAgent('public',
+                                 'public-authpass', 'public-privpass')
 
     def setUp(self):
-        self.session = snmp.Session(
-            host="127.0.0.1:{0}".format(self.agent.port),
-            community="public",
-            version=self.version)
+        params = self.setUpSession(self.agent, 'public')
+        self.session = snmp.Session(**params)
+
+    def setUpSession(self, agent, password):
+        return dict(host="127.0.0.1:{0}".format(agent.port),
+                    community=password,
+                    version=self.version)
 
     @classmethod
     def tearDownClass(cls):
-        cls.agent.terminate()
+        for a in cls.agents:
+            a.terminate()
 
     def testGetString(self):
         """Get a string value"""
         ooid = mib.get('SNMPv2-MIB', 'sysDescr').oid + (0,)
         oid, a = self.session.get(ooid)[0]
         self.assertEqual(oid, ooid)
-        self.assertEqual(a, b"Snimpy Test Agent")
+        self.assertEqual(a, b"Snimpy Test Agent public")
 
     def testGetInteger(self):
         """Get an integer value"""
@@ -170,10 +186,12 @@ class TestSnmp1(unittest.TestCase):
 
     def testInexistant(self):
         """Get an inexistant value"""
-        self.assertRaises(
-            self.version == 1 and snmp.SNMPNoSuchName or snmp.SNMPNoSuchObject,
-            self.session.get,
-            (1, 2, 3))
+        try:
+            self.session.get((1, 2, 3))
+            self.assertFalse("we should have got an exception")
+        except snmp.SNMPException as ex:
+            self.assert_(isinstance(ex, snmp.SNMPNoSuchName) or
+                         isinstance(ex, snmp.SNMPNoSuchObject))
 
     def testSetIpAddress(self):
         """Set IpAddress."""
@@ -243,7 +261,7 @@ class TestSnmp1(unittest.TestCase):
         self.assertEqual(oid1, ooid1)
         self.assertEqual(oid2, ooid2)
         self.assertEqual(oid3, ooid3)
-        self.assertEqual(a1, b"Snimpy Test Agent")
+        self.assertEqual(a1, b"Snimpy Test Agent public")
         self.assert_(a2 > 1)
         b = basictypes.build('IF-MIB', 'ifType', a3)
         self.assertEqual(b, "softwareLoopback")
@@ -272,6 +290,60 @@ class TestSnmp1(unittest.TestCase):
                           (ooid + (2,), b"eth0"),
                           (ooid + (3,), b"eth1")))
 
+    def testSeveralSessions(self):
+        """Test with two sessions"""
+        agent2 = self.addAgent('private',
+                               'private-authpass', 'private-privpass')
+        params = self.setUpSession(agent2, 'private')
+        session2 = snmp.Session(**params)
+
+        ooid = mib.get('SNMPv2-MIB', 'sysDescr').oid + (0,)
+        oid1, a1 = self.session.get(ooid)[0]
+        oid2, a2 = session2.get(ooid)[0]
+        self.assertEqual(oid1, ooid)
+        self.assertEqual(oid2, ooid)
+        self.assertEqual(a1, b"Snimpy Test Agent public")
+        self.assertEqual(a2, b"Snimpy Test Agent private")
+
+    def testMultipleThreads(self):
+        """Test with multiple sessions in different threads."""
+        count = 20
+        agents = []
+        for i in range(count):
+            agents.append(self.addAgent('community{}'.format(i),
+                                        'community{}-authpass'.format(i),
+                                        'community{}-privpass'.format(i)))
+        ooid = mib.get('SNMPv2-MIB', 'sysDescr').oid + (0,)
+
+        threads = []
+        successes = []
+        failures = []
+        lock = multiprocessing.Lock()
+
+        # Start one thread
+        def run(i):
+            params = self.setUpSession(agents[i], 'community{}'.format(i))
+            session = snmp.Session(**params)
+            session.timeout = 10 * 1000 * 1000
+            oid, a = session.get(ooid)[0]
+            exp = ("Snimpy Test Agent community{}".format(i)).encode('ascii')
+            with lock:
+                if oid == ooid and \
+                   a == exp:
+                    successes.append("community{}".format(i))
+                else:
+                    failures.append("community{}".format(i))
+        for i in range(count):
+            threads.append(threading.Thread(target=run, args=(i,)))
+        for i in range(count):
+            threads[i].start()
+        for i in range(count):
+            threads[i].join()
+        self.assertEqual(failures, [])
+        self.assertEqual(sorted(successes),
+                         sorted(["community{}".format(i)
+                                 for i in range(count)]))
+
 
 class TestSnmp2(TestSnmp1):
 
@@ -287,11 +359,10 @@ class TestSnmp2(TestSnmp1):
         ooid = mib.get("IF-MIB", "ifDescr").oid
         self.session.bulk = 4
         results = self.session.walk(ooid)
-        self.assertEqual(results,
+        self.assertEqual(results[:3],
                          ((ooid + (1,), b"lo"),
                           (ooid + (2,), b"eth0"),
-                          (ooid + (3,), b"eth1"),
-                          (mib.get("IF-MIB", "ifType").oid + (1,), 24)))
+                          (ooid + (3,), b"eth1")))
         self.session.bulk = 2
         results = self.session.walk(ooid)
         self.assertEqual(results[:2],
@@ -304,13 +375,14 @@ class TestSnmp3(TestSnmp2):
     """Test communicaton with an agent with SNMPv3."""
     version = 3
 
-    def setUp(self):
-        self.session = snmp.Session(
-            host="127.0.0.1:{0}".format(self.agent.port),
-            version=3,
-            secname="read-write",
-            authprotocol="MD5", authpassword="authpass",
-            privprotocol="AES", privpassword="privpass")
+    def setUpSession(self, agent, password):
+        return dict(host="127.0.0.1:{0}".format(agent.port),
+                    version=3,
+                    secname="read-write",
+                    authprotocol="MD5",
+                    authpassword="{0}-authpass".format(password),
+                    privprotocol="AES",
+                    privpassword="{0}-privpass".format(password))
 
 
 class TestSnmpTransports(unittest.TestCase):
@@ -332,7 +404,7 @@ class TestSnmpTransports(unittest.TestCase):
         try:
             ooid = mib.get('SNMPv2-MIB', 'sysDescr').oid + (0,)
             oid, a = session.get(ooid)[0]
-            self.assertEqual(a, b"Snimpy Test Agent")
+            self.assertEqual(a, b"Snimpy Test Agent public")
         finally:
             m.terminate()
 
